"""Constants for the Sagemcom F@st client."""
API_ENDPOINT = "/cgi/json-req"

DEFAULT_TIMEOUT = 7
DEFAULT_USER_AGENT = "Python_Sagemcom"

XMO_ACCESS_RESTRICTION_ERR = "XMO_ACCESS_RESTRICTION_ERR"
XMO_AUTHENTICATION_ERR = "XMO_AUTHENTICATION_ERR"
XMO_NON_WRITABLE_PARAMETER_ERR = "XMO_NON_WRITABLE_PARAMETER_ERR"
XMO_NO_ERR = "XMO_NO_ERR"
XMO_REQUEST_ACTION_ERR = "XMO_REQUEST_ACTION_ERR"
XMO_REQUEST_NO_ERR = "XMO_REQUEST_NO_ERR"
XMO_UNKNOWN_PATH_ERR = "XMO_UNKNOWN_PATH_ERR"
